#include "..\Plugin API\ExtAPI.h"
#include "zProject.h"
#include "..\Plugin API\zClasses.h"
#include "..\Plugin API\zFunc.h"

extern CExtAPI theAPI;

//
// init state
//

bool zproj_IsInit() {
	CzVar v;
	v.Create(VarID_bool);
	if(!theAPI.script_Execute("zs", "project.IsInit", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zproj_Init() {
	CzVar v;
	v.Create(VarID_bool);
	if(!theAPI.script_Execute("zs", "project.InitProject", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zproj_Free() {
	CzVar v;
	v.Create(VarID_bool);
	if(!theAPI.script_Execute("zs", "project.FreeProject", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}

// loading/saving project

bool zproj_LoadProject(const char* lpProjName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.LoadProject \"");
	s.Append(lpProjName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zproj_SaveProject() {
	CzVar v;
	v.Create(VarID_bool);
	if(!theAPI.script_Execute("zs", "project.SaveProject", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zproj_SaveProjectFile() {		// just save the project file 
	CzVar v;
	v.Create(VarID_bool);
	if(!theAPI.script_Execute("zs", "project.SaveProjectFile", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zproj_SaveProjectAs(const char* lpProjName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.SaveProjectAs \"");
	s.Append(lpProjName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zproj_DemandSave() {
	CzStr s;
	if(!theAPI.script_Execute("zs", "project.SaveProjectAsUI", (ZVAR)s)) { 
		return false;
	}
	
	if(s.GetLength())
			return true;
	else	return false;
}

bool zproj_IsPathOK() { 
	if(strlen(zproj_GetProjectFilename()))
		return true;

	return false;
}
const char* zproj_GetProjectFilename() {
	return theAPI.project_GetProjectFilename();
}

//
// saving map
//

bool zproj_AutoSaveMap(const char* lpMapName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.AutoSaveMap \"");
	s.Append(lpMapName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zproj_AutoLoadMap(const char* lpMapName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.AutoLoadMap \"");
	s.Append(lpMapName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

//
// accessing maps
//

ZLIST zproj_GetMapList() {

	CzFunc f;
	if(!f.GetFunc("project.GetMapListH")) {
		theAPI.ReportError("zproj_GetMapList error:\r\n - can't get function handle");
		return NULL;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("zproj_GetMapList error:\r\n - extension function failed");
		return NULL;
	}

	ZVAR hRval;
	if(!f.m_rval.GetValue(VarID_hvar, &hRval)) {
		theAPI.ReportError("zproj_GetMapList error:\r\n - can't get return value");
		return NULL;
	}
	return hRval;
}
ZMAP zproj_GetMap(const char* lpMapName) {
	return theAPI.project_GetMap(lpMapName);
}
ZMAP zproj_CreateMap(const char* lpMapName) {
	return theAPI.project_CreateMap(lpMapName);
}
ZMAP zproj_GetOrCreateMap(const char* lpMapName) {
	return theAPI.project_GetOrCreateMap(lpMapName);
}

//
// settings and options
//

ZLIST zproj_GetSettingsList() {

	CzFunc f;
	if(!f.GetFunc("project.GetSettingsListH")) {
		theAPI.ReportError("zproj_GetSettingsList error:\r\n - can't get function handle");
		return NULL;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("zproj_GetSettingsList error:\r\n - extension function failed");
		return NULL;
	}

	ZVAR hRval;
	if(!f.m_rval.GetValue(VarID_hvar, &hRval)) {
		theAPI.ReportError("zproj_GetSettingsList error:\r\n - can't get return value");
		return NULL;
	}
	return hRval;
	
}
bool zproj_GetSetting(const char* lpName, long VarID, void* pValue) {
	ZLIST hSettings = zproj_GetSettingsList();
	if(!hSettings) {
		return false;
	}

	ZVAR hVar = theAPI.list_GetItemA(hSettings, lpName);
	if(!hVar) {
		return false;
	}

	if(!theAPI.var_GetValueEx(hVar, VarID, pValue)) {
		return false;
	}

	return true;
}

bool zproj_SetSetting(const char* lpName, long VarID, void* pValue) {
	ZLIST hSettings = zproj_GetSettingsList();
	if(!hSettings) {
		return false;
	}

	ZVAR hVar = theAPI.list_GetItemA(hSettings, lpName);
	if(!hVar) {
		return false;
	}

	if(!theAPI.var_SetValueEx(hVar, VarID, pValue)) {
		return false;
	}

	return true;
}

bool zproj_GetOpFlag(const char* lpOpName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.GetOpFlag \"");
	s.Append(lpOpName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zproj_SetOpFlag(const char* lpOpName, bool StateFlag) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.SetOpFlag \"");
	s.Append(lpOpName);
	s.Append("\" ");
	if(StateFlag)
			s.Append("true");
	else	s.Append("false");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zproj_DelOpsFollowing(const char* lpOpName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.DelOpsFollowing \"");
	s.Append(lpOpName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zproj_GetHeightfieldRange(float& minval, float& maxval) {
	return theAPI.project_GetHeightfieldRange(minval, maxval);
}

bool zproj_SetMapFlag(const char* lpMapName, long MapFlag, bool State) {
	ZMAP m = zproj_GetMap(lpMapName);
	if(!m) {
		return false;
	}

	return theAPI.map_SetFlag(m, MapFlag, State);
}


long zproj_GetCurTileSizeDM() {
	
	CzFunc f;
	f.GetFunc("project.GetCurTileSizeDM");
	f.Execute();

	long rval;
	f.m_rval.GetValue(VarID_int, &rval);

	return rval;
}
bool zproj_SetCurTileSizeDM(long TileSizeDM) {

	CzFunc f;
	f.GetFunc("project.SetCurTileSizeDM");

	f.m_args.SetItemValueI(0, VarID_int, &TileSizeDM);
	f.Execute();

	bool rval;
	f.m_rval.GetValue(VarID_bool, &rval);
	return rval;
}

float zproj_GetCurHorizScaleHF() {

	CzFunc f;
	f.GetFunc("project.GetCurHorizScaleHF");
	f.Execute();

	float rval;
	f.m_rval.GetValue(VarID_float, &rval);

	return rval;
}
bool zproj_SetCurHorizScaleHF(float HorizScaleHF) {

	CzFunc f;
	f.GetFunc("project.SetCurHorizScaleHF");

	f.m_args.SetItemValue((long)0, VarID_float, &HorizScaleHF);
	f.Execute();

	bool rval;
	f.m_rval.GetValue(VarID_bool, &rval);
	return rval;
}